@@ -364,7 +364,9 @@ SEXP rnng_stream_dial(SEXP, SEXP, SEXP);
 SEXP rnng_stream_listen(SEXP, SEXP, SEXP);
 SEXP rnng_strerror(SEXP);
 SEXP rnng_subscribe(SEXP, SEXP, SEXP);
+SEXP rnng_thread_shutdown(void);
 SEXP rnng_tls_config(SEXP, SEXP, SEXP, SEXP);
+SEXP rnng_traverse_precious(void);
 SEXP rnng_unresolved(SEXP);
 SEXP rnng_unresolved2(SEXP);
 SEXP rnng_url_parse(SEXP);
