@@ -65,32 +65,32 @@ SEXP rnng_protocol_open(SEXP protocol, SEXP dial, SEXP listen, SEXP tls, SEXP au
   case 1:
   case 2:
   case 3:
-    if (!strncmp(pro, "bus", slen)) {
+    if (!memcmp(pro, "bus", slen)) {
       pname = "bus";
       sock = R_Calloc(1, nng_socket);
       xc = rw ? nng_bus0_open_raw(sock) : nng_bus0_open(sock);
       break;
     }
     if (slen > 2) {
-      if (!strncmp(pro, "pub", slen)) {
+      if (!memcmp(pro, "pub", slen)) {
         pname = "pub";
         sock = R_Calloc(1, nng_socket);
         xc = rw ? nng_pub0_open_raw(sock) : nng_pub0_open(sock);
         break;
       }
-      if (!strncmp(pro, "sub", slen)) {
+      if (!memcmp(pro, "sub", slen)) {
         pname = "sub";
         sock = R_Calloc(1, nng_socket);
         xc = rw ? nng_sub0_open_raw(sock) : nng_sub0_open(sock);
         break;
       }
-      if (!strncmp(pro, "req", slen)) {
+      if (!memcmp(pro, "req", slen)) {
         pname = "req";
         sock = R_Calloc(1, nng_socket);
         xc = rw ? nng_req0_open_raw(sock) : nng_req0_open(sock);
         break;
       }
-      if (!strncmp(pro, "rep", slen)) {
+      if (!memcmp(pro, "rep", slen)) {
         pname = "rep";
         sock = R_Calloc(1, nng_socket);
         xc = rw ? nng_rep0_open_raw(sock) : nng_rep0_open(sock);
@@ -99,26 +99,26 @@ SEXP rnng_protocol_open(SEXP protocol, SEXP dial, SEXP listen, SEXP tls, SEXP au
     }
   case 4:
     if (slen > 1) {
-      if (!strncmp(pro, "pair", slen)) {
+      if (!memcmp(pro, "pair", slen)) {
         pname = "pair";
         sock = R_Calloc(1, nng_socket);
         xc = rw ? nng_pair0_open_raw(sock) : nng_pair0_open(sock);
         break;
       }
-      if (!strncmp(pro, "poly", slen)) {
+      if (!memcmp(pro, "poly", slen)) {
         pname = "poly";
         sock = R_Calloc(1, nng_socket);
         xc = rw ? nng_pair1_open_raw(sock) : nng_pair1_open_poly(sock);
         break;
       }
       if (slen > 2) {
-        if (!strncmp(pro, "push", slen)) {
+        if (!memcmp(pro, "push", slen)) {
           pname = "push";
           sock = R_Calloc(1, nng_socket);
           xc = rw ? nng_push0_open_raw(sock) : nng_push0_open(sock);
           break;
         }
-        if (!strncmp(pro, "pull", slen)) {
+        if (!memcmp(pro, "pull", slen)) {
           pname = "pull";
           sock = R_Calloc(1, nng_socket);
           xc = rw ? nng_pull0_open_raw(sock) : nng_pull0_open(sock);
@@ -130,7 +130,7 @@ SEXP rnng_protocol_open(SEXP protocol, SEXP dial, SEXP listen, SEXP tls, SEXP au
   case 6:
   case 7:
   case 8:
-    if (slen > 2 && !strncmp(pro, "surveyor", slen)) {
+    if (slen > 2 && !memcmp(pro, "surveyor", slen)) {
       pname = "surveyor";
       sock = R_Calloc(1, nng_socket);
       xc = rw ? nng_surveyor0_open_raw(sock) : nng_surveyor0_open(sock);
@@ -138,7 +138,7 @@ SEXP rnng_protocol_open(SEXP protocol, SEXP dial, SEXP listen, SEXP tls, SEXP au
     }
   case 9:
   case 10:
-    if (slen > 2 && !strncmp(pro, "respondent", slen)) {
+    if (slen > 2 && !memcmp(pro, "respondent", slen)) {
       pname = "respondent";
       sock = R_Calloc(1, nng_socket);
       xc = rw ? nng_respondent0_open_raw(sock) : nng_respondent0_open(sock);
@@ -223,7 +223,7 @@ SEXP rnng_aio_collect_pipe(SEXP aio) {
 
   if (TYPEOF(aio) != ENVSXP)
     goto exitlevel1;
-  const SEXP coreaio = Rf_findVarInFrame(aio, nano_AioSymbol);
+  const SEXP coreaio = nano_findVarInFrame(aio, nano_AioSymbol);
   if (NANO_TAG(coreaio) != nano_AioSymbol)
     goto exitlevel1;
 
