@@ -20,11 +20,40 @@
 #define NANONEXT_IO
 #include "nanonext.h"
 
-// messenger -------------------------------------------------------------------
+// threads callable and messenger ----------------------------------------------
+
+extern nng_thread *nano_wait_thr;
+extern nng_aio *nano_shared_aio;
+extern nng_mtx *nano_wait_mtx;
+extern nng_cv *nano_wait_cv;
+extern int nano_wait_condition;
 
 // # nocov start
 // tested interactively
 
+static void thread_finalizer(SEXP xptr) {
+
+  if (NANO_PTR(xptr) == NULL) return;
+  nng_thread *xp = (nng_thread *) NANO_PTR(xptr);
+  nng_thread_destroy(xp);
+
+}
+
+static SEXP rnng_thread_create(void (*func)(void *), void *arg) {
+
+  nng_thread *thr;
+  int xc;
+
+  if ((xc = nng_thread_create(&thr, func, arg)))
+    ERROR_OUT(xc);
+
+  SEXP xptr = R_MakeExternalPtr(thr, R_NilValue, R_NilValue);
+  R_RegisterCFinalizerEx(xptr, thread_finalizer, TRUE);
+
+  return xptr;
+
+}
+
 static void nano_printf(const int err, const char *fmt, ...) {
 
   char buf[NANONEXT_INIT_BUFSIZE];
@@ -38,14 +67,6 @@ static void nano_printf(const int err, const char *fmt, ...) {
 
 }
 
-static void thread_finalizer(SEXP xptr) {
-
-  if (NANO_PTR(xptr) == NULL) return;
-  nng_thread *xp = (nng_thread *) NANO_PTR(xptr);
-  nng_thread_destroy(xp);
-
-}
-
 static void rnng_messenger_thread(void *args) {
 
   SEXP plist = (SEXP) args;
@@ -126,13 +147,13 @@ SEXP rnng_messenger(SEXP url) {
   if ((xc = nng_listen(*sock, up, lp, 0))) {
     if (xc != 10 && xc != 15) {
       R_Free(lp);
-      goto exitlevel1;
+      goto exitlevel2;
     }
     R_Free(lp);
     dp = R_Calloc(1, nng_dialer);
     if ((xc = nng_dial(*sock, up, dp, 0))) {
       R_Free(dp);
-      goto exitlevel1;
+      goto exitlevel2;
     }
     dialer = 1;
   }
@@ -151,6 +172,8 @@ SEXP rnng_messenger(SEXP url) {
   UNPROTECT(2);
   return socket;
 
+  exitlevel2:
+  nng_close(*sock);
   exitlevel1:
   R_Free(sock);
   ERROR_OUT(xc);
@@ -160,15 +183,7 @@ SEXP rnng_messenger(SEXP url) {
 SEXP rnng_messenger_thread_create(SEXP args) {
 
   SEXP socket = CADR(args);
-  nng_thread *thr;
-
-  const int xc = nng_thread_create(&thr, rnng_messenger_thread, args);
-  if (xc)
-    ERROR_OUT(xc);
-
-  SEXP xptr = R_MakeExternalPtr(thr, R_NilValue, R_NilValue);
-  NANO_SET_PROT(socket, xptr);
-  R_RegisterCFinalizerEx(xptr, thread_finalizer, TRUE);
+  NANO_SET_PROT(socket, rnng_thread_create(rnng_messenger_thread, args));
 
   return socket;
 
@@ -212,22 +227,6 @@ static void thread_duo_finalizer(SEXP xptr) {
 
 }
 
-static void rnng_wait_thread(void *args) {
-
-  nano_thread_aio *taio = (nano_thread_aio *) args;
-  nano_cv *ncv = taio->cv;
-  nng_mtx *mtx = ncv->mtx;
-  nng_cv *cv = ncv->cv;
-
-  nng_aio_wait(taio->aio);
-
-  nng_mtx_lock(mtx);
-  ncv->condition = 1;
-  nng_cv_wake(cv);
-  nng_mtx_unlock(mtx);
-
-}
-
 static void thread_disp_finalizer(SEXP xptr) {
 
   if (NANO_PTR(xptr) == NULL) return;
@@ -239,9 +238,8 @@ static void thread_disp_finalizer(SEXP xptr) {
   ncv->condition = -1;
   nng_cv_wake(cv);
   nng_mtx_unlock(mtx);
-  if (xp->tls != NULL) {
+  if (xp->tls != NULL)
     nng_tls_config_free(xp->tls);
-  }
   nng_thread_destroy(xp->thr);
   nng_url_free(xp->up);
   for (int i = 0; i < xp->n; i++) {
@@ -265,6 +263,102 @@ static void thread_disp_finalizer(SEXP xptr) {
 
 }
 
+static void rnng_wait_thread(void *args) {
+
+  while (1) {
+    nng_mtx_lock(nano_wait_mtx);
+    while (nano_wait_condition == 0)
+      nng_cv_wait(nano_wait_cv);
+    if (nano_wait_condition == -1) {
+      nng_mtx_unlock(nano_wait_mtx);
+      break;
+    }
+    nng_mtx_unlock(nano_wait_mtx);
+
+    nng_aio_wait(nano_shared_aio);
+
+    nng_mtx_lock(nano_wait_mtx);
+    nano_shared_aio = NULL;
+    nano_wait_condition = 0;
+    nng_cv_wake(nano_wait_cv);
+    nng_mtx_unlock(nano_wait_mtx);
+  }
+
+}
+
+static void rnng_wait_thread_single(void *args) {
+
+  nano_thread_aio *taio = (nano_thread_aio *) args;
+  nano_cv *ncv = taio->cv;
+  nng_mtx *mtx = ncv->mtx;
+  nng_cv *cv = ncv->cv;
+
+  nng_aio_wait(taio->aio);
+
+  nng_mtx_lock(mtx);
+  ncv->condition = 1;
+  nng_cv_wake(cv);
+  nng_mtx_unlock(mtx);
+
+}
+
+void single_wait_thread_create(SEXP x) {
+
+  nano_aio *aiop = (nano_aio *) NANO_PTR(x);
+  nano_thread_aio *taio = R_Calloc(1, nano_thread_aio);
+  nano_cv *ncv = R_Calloc(1, nano_cv);
+  taio->aio = aiop->aio;
+  taio->cv = ncv;
+  nng_mtx *mtx;
+  nng_cv *cv;
+  int xc, signalled;
+
+  if ((xc = nng_mtx_alloc(&mtx)))
+    goto exitlevel1;
+
+  if ((xc = nng_cv_alloc(&cv, mtx)))
+    goto exitlevel2;
+
+  ncv->mtx = mtx;
+  ncv->cv = cv;
+
+  if ((xc = nng_thread_create(&taio->thr, rnng_wait_thread_single, taio)))
+    goto exitlevel3;
+
+  SEXP xptr;
+  PROTECT(xptr = R_MakeExternalPtr(taio, R_NilValue, R_NilValue));
+  R_RegisterCFinalizerEx(xptr, thread_aio_finalizer, TRUE);
+  R_MakeWeakRef(x, xptr, R_NilValue, TRUE);
+  UNPROTECT(1);
+
+  nng_time time = nng_clock();
+
+  while (1) {
+    time = time + 400;
+    signalled = 1;
+    nng_mtx_lock(mtx);
+    while (ncv->condition == 0) {
+      if (nng_cv_until(cv, time) == NNG_ETIMEDOUT) {
+        signalled = 0;
+        break;
+      }
+    }
+    nng_mtx_unlock(mtx);
+    if (signalled) break;
+    R_CheckUserInterrupt();
+  }
+
+  return;
+
+  exitlevel3:
+  nng_cv_free(cv);
+  exitlevel2:
+  nng_mtx_free(mtx);
+  exitlevel1:
+  ERROR_OUT(xc);
+
+}
+
 SEXP rnng_wait_thread_create(SEXP x) {
 
   const SEXPTYPE typ = TYPEOF(x);
@@ -274,51 +368,57 @@ SEXP rnng_wait_thread_create(SEXP x) {
     if (NANO_TAG(coreaio) != nano_AioSymbol)
       return x;
 
-    PROTECT(coreaio);
     nano_aio *aiop = (nano_aio *) NANO_PTR(coreaio);
 
-    nano_thread_aio *taio = R_Calloc(1, nano_thread_aio);
-    nano_cv *ncv = R_Calloc(1, nano_cv);
-    taio->aio = aiop->aio;
-    taio->cv = ncv;
-    nng_mtx *mtx;
-    nng_cv *cv;
-
     int xc, signalled;
 
-    if ((xc = nng_mtx_alloc(&mtx)))
-      goto exitlevel1;
+    if (!nano_wait_thr) {
+      if ((xc = nng_mtx_alloc(&nano_wait_mtx)))
+        goto exitlevel1;
 
-    if ((xc = nng_cv_alloc(&cv, mtx)))
-      goto exitlevel2;
+      if ((xc = nng_cv_alloc(&nano_wait_cv, nano_wait_mtx)))
+        goto exitlevel2;
 
-    ncv->mtx = mtx;
-    ncv->cv = cv;
+      if ((xc = nng_thread_create(&nano_wait_thr, rnng_wait_thread, NULL)))
+        goto exitlevel3;
+    }
 
-    if ((xc = nng_thread_create(&taio->thr, rnng_wait_thread, taio)))
-      goto exitlevel3;
+    int thread_required = 0;
+    nng_mtx_lock(nano_wait_mtx);
+    if (nano_wait_condition == 0) {
+      nano_shared_aio = aiop->aio;
+      nano_wait_condition = 1;
+      nng_cv_wake(nano_wait_cv);
+    } else {
+      thread_required = nano_shared_aio != aiop->aio;
+    }
+    nng_mtx_unlock(nano_wait_mtx);
 
-    SEXP xptr;
-    PROTECT(xptr = R_MakeExternalPtr(taio, R_NilValue, R_NilValue));
-    R_RegisterCFinalizerEx(xptr, thread_aio_finalizer, TRUE);
-    R_MakeWeakRef(coreaio, xptr, R_NilValue, TRUE);
-    UNPROTECT(2);
+    if (thread_required) {
 
-    nng_time time = nng_clock();
+      PROTECT(coreaio);
+      single_wait_thread_create(coreaio);
+      UNPROTECT(1);
 
-    while (1) {
-      time = time + 400;
-      signalled = 1;
-      nng_mtx_lock(mtx);
-      while (ncv->condition == 0) {
-        if (nng_cv_until(cv, time) == NNG_ETIMEDOUT) {
-          signalled = 0;
-          break;
+    } else {
+
+      nng_time time = nng_clock();
+
+      while (1) {
+        time = time + 400;
+        signalled = 1;
+        nng_mtx_lock(nano_wait_mtx);
+        while (nano_wait_condition == 1) {
+          if (nng_cv_until(nano_wait_cv, time) == NNG_ETIMEDOUT) {
+            signalled = 0;
+            break;
+          }
         }
+        nng_mtx_unlock(nano_wait_mtx);
+        if (signalled) break;
+        R_CheckUserInterrupt();
       }
-      nng_mtx_unlock(mtx);
-      if (signalled) break;
-      R_CheckUserInterrupt();
+
     }
 
     switch (aiop->type) {
@@ -342,12 +442,10 @@ SEXP rnng_wait_thread_create(SEXP x) {
     return x;
 
     exitlevel3:
-    nng_cv_free(cv);
+    nng_cv_free(nano_wait_cv);
     exitlevel2:
-    nng_mtx_free(mtx);
+    nng_mtx_free(nano_wait_mtx);
     exitlevel1:
-    R_Free(ncv);
-    R_Free(taio);
     ERROR_OUT(xc);
 
   } else if (typ == VECSXP) {
@@ -363,6 +461,21 @@ SEXP rnng_wait_thread_create(SEXP x) {
 
 }
 
+SEXP rnng_thread_shutdown(void) {
+  if (nano_wait_thr != NULL) {
+    if (nano_shared_aio != NULL)
+      nng_aio_stop(nano_shared_aio);
+    nng_mtx_lock(nano_wait_mtx);
+    nano_wait_condition = -1;
+    nng_cv_wake(nano_wait_cv);
+    nng_mtx_unlock(nano_wait_mtx);
+    nng_thread_destroy(nano_wait_thr);
+    nng_cv_free(nano_wait_cv);
+    nng_mtx_free(nano_wait_mtx);
+  }
+  return R_NilValue;
+}
+
 static void nano_record_pipe(nng_pipe p, nng_pipe_ev ev, void *arg) {
 
   nano_signal *signal = (nano_signal *) arg;
