@@ -480,7 +480,7 @@ is_nano(s) && {
   test_type("integer", send(s, c("message1", "test"), block = 500L))
   test_notnull(recv(s, block = FALSE))
   test_type("integer", send(s, "message2", block = FALSE))
-  test_notnull(recv(s, mode = 9L, block = 100))
+  test_notnull(suppressWarnings(recv(s, mode = 9L, block = 100)))
   test_type("integer", send(s, 2L, block = 500))
   test_class("recvAio", sr <- recv_aio(s, mode = "i", timeout = 500L, n = 8192L))
   test_notnull(suppressWarnings(call_aio(sr)[["data"]]))
@@ -539,7 +539,8 @@ test_zero(send(s, TRUE, block = 500L))
 test_true(recv(disp, block = 500L))
 test_zero(send(disp, NULL, block = 500L))
 test_null(recv(s, block = 500L))
-test_zero(close(s))
+test_zero(reap(s))
+Sys.sleep(0.1)
 test_class("nanoSocket", s <- socket(protocol = "rep"))
 test_zero(pipe_notify(s, cv, add = TRUE))
 test_zero(dial(s, url = "inproc://disp/1"))
@@ -550,7 +551,7 @@ test_true(.mark())
 test_zero(send(s, NULL, block = 500L))
 test_true(!.mark(FALSE))
 test_null(recv(disp, block = 500L))
-test_zero(close(s))
+test_zero(reap(s))
 rm(disp)
 
 test_equal(nanonext:::.DollarNames.ncurlAio(NULL, "sta"), "status")
